/***********************************************
 *	FILE:	Receiver_AeroQuadMega.cpp
 *
 *	AUTHORS:	-Guillermo Maroto
 *	Based on Aeroquad.com/Receiver.h
 *           
 *   --------+--------         --------+--------
 *          | |_______________________| | 
 *                   |         |
 *                   |_________|
 *                 ~____|___|____    
 **********************************************/
#include "Receiver_AeroQuadMega.h"

/******************************************************/
/*************** AeroQuad Mega PCINT ******************/
/******************************************************/
volatile uint8_t *port_to_pcmask[] = {
&PCMSK0,
&PCMSK1,
&PCMSK2
};
volatile static uint8_t PCintLast[3];
// Channel data
typedef struct {
	byte edge;
	unsigned long riseTime;
	unsigned long fallTime;
	unsigned int lastGoodWidth;
} tPinTimingData;
volatile static tPinTimingData pinData[LASTCHANNEL];

static void MegaPcIntISR() {
	uint8_t bit;
	uint8_t curr;
	uint8_t mask;
	uint8_t pin;
	uint32_t currentTime;
	uint32_t time;
	
	curr = *portInputRegister(11);
	mask = curr ^ PCintLast[0];
	PCintLast[0] = curr;
	
	// mask is pins that have changed. screen out non pcint pins.
	if ((mask &= PCMSK2) == 0) {
		return;
	}
	
	currentTime = micros();
	
	// mask is pcint pins that have changed.
	for (uint8_t i=0; i < 8; i++) {
		bit = 0x01 << i;
		if (bit & mask) {
			pin = i;
			// for each pin changed, record time of change
			if (bit & PCintLast[0]) {
				time = currentTime - pinData[pin].fallTime;
				pinData[pin].riseTime = currentTime;
				if ((time >= MINOFFWIDTH) && (time <= MAXOFFWIDTH))
					pinData[pin].edge = RISING_EDGE;
				else
					pinData[pin].edge = FALLING_EDGE; // invalid rising edge detected
			}
			else {
				time = currentTime - pinData[pin].riseTime;
				pinData[pin].fallTime = currentTime;
				if ((time >= MINONWIDTH) && (time <= MAXONWIDTH) && (pinData[pin].edge == RISING_EDGE)) {
					pinData[pin].lastGoodWidth = time;
					pinData[pin].edge = FALLING_EDGE;
				}
			}
		}
	}
}

SIGNAL(PCINT2_vect) {
	MegaPcIntISR();
}


//arduino pins 63, 64, 65, 62, 66, 67
static byte receiverPin[8] = {1, 2, 3, 0, 4, 5, 6, 7}; // bit number of PORTK used for ROLL, PITCH, YAW, THROTTLE, MODE, AUX


void Receiver_AeroQuadMega::initialize() {
	this->_initialize(); // load in calibration xmitFactor from EEPROM
	DDRK = 0;
	PORTK = 0;
	PCMSK2 |= 0xFF;
	PCICR |= 0x1 << 2;
	
	for (byte channel = ROLL; channel < LASTCHANNEL; channel++)
		pinData[receiverPin[channel]].edge = FALLING_EDGE;
}

// Calculate PWM pulse width of receiver data
// If invalid PWM measured, use last known good time
void Receiver_AeroQuadMega::read(void) {
	for(byte channel = ROLL; channel < LASTCHANNEL; channel++) {
		byte pin = receiverPin[channel];
		uint8_t oldSREG = SREG;
		cli();
		// Get receiver value read by pin change interrupt handler
		uint16_t lastGoodWidth = pinData[pin].lastGoodWidth;
		SREG = oldSREG;
		
		// Apply transmitter calibration adjustment
		receiverData[channel] = (mTransmitter[channel] * lastGoodWidth) + bTransmitter[channel];
		// Smooth the flight control transmitter inputs
		transmitterCommandSmooth[channel] = filterSmooth(receiverData[channel], transmitterCommandSmooth[channel], transmitterSmooth[channel]);
	}
	
	// Reduce transmitter commands using xmitFactor and center around 1500
	for (byte channel = ROLL; channel < LASTCHANNEL; channel++)
		if (channel < THROTTLE)
			transmitterCommand[channel] = ((transmitterCommandSmooth[channel] - transmitterZero[channel]) * xmitFactor) + transmitterZero[channel];
		else
			// No xmitFactor reduction applied for throttle, mode and AUX
			transmitterCommand[channel] = transmitterCommandSmooth[channel];
	//transmitterCommand[THROTTLE] = (transmitterCommand[THROTTLE] * 0.8) + 200; // lower max throttle to allow for motor control
}